require 'spec_helper'

describe 'puppet_enterprise::mcollective::client', :type => :define do
  context 'on a supported platform' do
    before :each do
      @facter_facts = {
        :osfamily                => 'RedHat',
        :lsbmajdistrelease       => '6',
        :puppetversion           => '3.6.2 (Puppet Enterprise 3.3.0)',
        :is_pe                   => 'true',
        :clientcert              => 'awesomecert'
      }

      @params = {
        :activemq_brokers => ['testagent'],
        :keypair_name     => 'pe-internal-puppet-console-mcollective-client',
        :create_user      => true,
        :home_dir         => '/opt/puppet/share/puppet-dashboard',
        :logfile          => '/var/log/pe-puppet-dashboard/mcollective_client.log',
        :stomp_password   => 'supersecret',
        :stomp_port       => '12345',
        :stomp_user       => 'anyone',
      }
    end

    let(:title) { 'puppet-dashboard' }
    let(:facts) { @facter_facts }
    let(:params) { @params }

    context "not creating user" do
      before(:each) { @params[:create_user] = false }
      it { should_not contain_user('puppet-dashboard').with('home' => '/opt/puppet/share/puppet-dashboard') }
    end

    context "using the default home dir" do
      before(:each) { @params.delete(:home_dir) }
      it { should contain_file('/var/lib/puppet-dashboard/.mcollective.d/awesomecert.private_key.pem') }
    end

    context "using a custom home dir" do
      context "creating the user" do
        it { should contain_user('puppet-dashboard').with('home' => '/opt/puppet/share/puppet-dashboard') }
        it { should contain_file('/opt/puppet/share/puppet-dashboard') }

        it { should contain_file('/opt/puppet/share/puppet-dashboard/.bashrc.custom').with(
          'ensure' => 'file',
          'owner'  => 'puppet-dashboard',
          'group'  => 'puppet-dashboard',
          'mode'   => '0600',
        )}

        it { should contain_file('/opt/puppet/share/puppet-dashboard/.mcollective.d/ca.cert.pem') }
        it { should contain_file('/opt/puppet/share/puppet-dashboard/.mcollective.d/awesomecert.cert.pem') }
        it { should contain_file('/opt/puppet/share/puppet-dashboard/.mcollective.d/awesomecert.private_key.pem') }
        it { should contain_file('/opt/puppet/share/puppet-dashboard/.mcollective.d/puppet-dashboard-private.pem') }

        context "client.cfg" do
          def should_have_setting(setting, value)
            should contain_file(client_cfg).with_content(%r[^#{Regexp.escape(setting)}\s*=\s*#{Regexp.escape(value)}$])
          end

          let(:client_cfg) { '/opt/puppet/share/puppet-dashboard/.mcollective' }

          it { should contain_file(client_cfg).with_mode('0600') }

          context "configuring the security provider" do
            it { should_have_setting('securityprovider', 'ssl') }
            it { should_have_setting('plugin.ssl_serializer', 'yaml') }

            it { should_have_setting('plugin.ssl_client_private', '/opt/puppet/share/puppet-dashboard/.mcollective.d/puppet-dashboard-private.pem') }
            it { should_have_setting('plugin.ssl_client_public', '/opt/puppet/share/puppet-dashboard/.mcollective.d/puppet-dashboard-public.pem') }
            it { should_have_setting('plugin.ssl_server_public', '/opt/puppet/share/puppet-dashboard/.mcollective.d/mcollective-public.pem') }
          end

          it { should_have_setting('main_collective', 'mcollective') }
          it { should_have_setting('factsource', 'yaml') }
          it { should_have_setting('plugin.yaml', '/etc/puppetlabs/mcollective/facts.yaml') }
          it { should_have_setting('libdir', '/opt/puppet/libexec/mcollective/') }
          it { should_have_setting('logfile', '/var/log/pe-puppet-dashboard/mcollective_client.log') }
          it { should_have_setting('loglevel', 'info') }

          context "managing the connector" do
            it { should_have_setting('connector', 'activemq') }

            context "with the default pool" do
              it { should_have_setting('plugin.activemq.pool.size', '1') }
              it { should_have_setting('plugin.activemq.pool.1.host', 'testagent') }
              it { should_have_setting('plugin.activemq.pool.1.port', '12345') }
              it { should_have_setting('plugin.activemq.pool.1.user', 'anyone') }
              it { should_have_setting('plugin.activemq.pool.1.password', 'supersecret') }
              it { should_have_setting('plugin.activemq.pool.1.ssl', 'true') }
              it { should_have_setting('plugin.ssl_client_private', '/opt/puppet/share/puppet-dashboard/.mcollective.d/puppet-dashboard-private.pem') }
              it { should_have_setting('plugin.ssl_client_public', '/opt/puppet/share/puppet-dashboard/.mcollective.d/puppet-dashboard-public.pem') }
              it { should_have_setting('plugin.ssl_server_public', '/opt/puppet/share/puppet-dashboard/.mcollective.d/mcollective-public.pem') }
            end

            context "specifying pool members with parameters" do
              before(:each) { @params[:activemq_brokers] = %w[foo bar] }

              it { should_have_setting('plugin.activemq.pool.size', '2') }

              %w[foo bar].each_with_index do |host, idx|
                i = idx + 1
                it { should_have_setting("plugin.activemq.pool.#{i}.host", host) }
                it { should_have_setting("plugin.activemq.pool.#{i}.port", '12345') }
                it { should_have_setting("plugin.activemq.pool.#{i}.user", 'anyone') }
                it { should_have_setting("plugin.activemq.pool.#{i}.password", 'supersecret') }
                it { should_have_setting("plugin.activemq.pool.#{i}.ssl", 'true') }
              end
            end
          end
        end
      end
    end
  end
end
